"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const get_between_1 = require("./get-between");
function countUsed(source, component) {
    const localSource = source.toString().replace(/(?:\r\n|\r|\n)/g, ' ');
    const items = [
        ...get_between_1.getBetween(localSource, `<${component.selector} `, `</${component.selector}>`),
        ...get_between_1.getBetween(localSource, `<${component.selector}>`, `</${component.selector}>`)
    ];
    return {
        allCount: items.length,
        withBindIOCount: items.filter(item => item.indexOf('bindIO') !== -1).length,
        withBindInputsCount: items.filter(item => item.indexOf('bindInputs') !== -1).length,
        withBindOutputsCount: items.filter(item => item.indexOf('bindOutputs') !== -1).length
    };
}
exports.countUsed = countUsed;
function parseArguments(dec) {
    let selector = '';
    let args = [];
    args = dec.getStructure().arguments;
    const parseds = args && args.length > 0 ? args[0].replace(new RegExp('\n', 'g'), '').
        replace(new RegExp(' ', 'g'), '').
        replace(new RegExp('{', 'g'), '').split(`'`) : [];
    parseds.forEach((parsed, index) => {
        if (index > 0 && parseds[index - 1].indexOf('selector:') !== -1) {
            selector = parsed;
        }
    });
    return {
        selector,
        arguments: args
    };
}
exports.parseArguments = parseArguments;
function updateAngularComponents(component) {
    const componentClass = component.class;
    const importDeclaration = component.sourceFile.getImportDeclaration((importDeclaration) => {
        const structure = importDeclaration.getStructure();
        if (structure.moduleSpecifier === 'ngx-binf-io' &&
            structure && structure.namedImports &&
            structure.namedImports.filter(namedImport => namedImport.name === 'BindIoInner').length > 0) {
            return true;
        }
        else {
            return false;
        }
    });
    if (!importDeclaration) {
        component.sourceFile.addImportDeclaration({
            defaultImport: undefined,
            moduleSpecifier: 'ngx-binf-io',
            namedImports: [{
                    name: 'BindIoInner',
                    alias: undefined
                }]
        });
        component.sourceFile.saveSync();
    }
    componentClass.getDecorators().map(classDecorator => {
        if (classDecorator.getName() === 'Component') {
            const baseClasses = [];
            let eachClass = componentClass;
            while (eachClass) {
                baseClasses.push(eachClass);
                eachClass = eachClass.getBaseClass();
            }
            component.inputs = [];
            baseClasses.forEach(derivedClasse => {
                const decoratedWithBindIoInner = derivedClasse.getDecorators().filter(dec => dec.getName() === 'BindIoInner').length > 0;
                const decoratedWithComponent = derivedClasse.getDecorators().filter(dec => dec.getName() === 'Component').length > 0;
                if (!decoratedWithBindIoInner && decoratedWithComponent) {
                    derivedClasse.insertDecorator(0, { name: 'BindIoInner', arguments: [] });
                }
                derivedClasse.getSourceFile().saveSync();
                derivedClasse.getInstanceProperties().forEach(prop => {
                    const propName = prop.getName() || '';
                    const propDeclaration = derivedClasse.getProperty(propName);
                    prop.getDecorators().filter(dec => {
                        const propDecName = dec.getName() || '';
                        if (propDecName === 'Input') {
                            let initialized = prop.getStructure()
                                .initializer !== undefined;
                            if (propDeclaration && !initialized) {
                                propDeclaration.setInitializer('undefined');
                                initialized = true;
                                propDeclaration.getSourceFile().saveSync();
                                component.autoInitializedInputs.push({
                                    name: propName,
                                    initialized: initialized
                                });
                            }
                            component.inputs.push({
                                name: propName,
                                initialized: initialized
                            });
                        }
                    });
                });
            });
        }
    });
    return component;
}
exports.updateAngularComponents = updateAngularComponents;
function getAngularComponents(sourceFile) {
    const components = [];
    const classes = sourceFile.getClasses();
    classes.forEach(foundedClass => {
        foundedClass.getDecorators().map(classDecorator => {
            if (classDecorator.getName() === 'Component') {
                const args = parseArguments(classDecorator);
                const component = {
                    sourceFile: sourceFile,
                    class: foundedClass,
                    selector: args.selector,
                    autoInitializedInputs: [],
                    inputs: [],
                    outputs: []
                };
                const baseClasses = [];
                let eachClass = foundedClass;
                while (eachClass) {
                    baseClasses.push(eachClass);
                    eachClass = eachClass.getBaseClass();
                }
                baseClasses.forEach(derivedClasse => derivedClasse.getInstanceProperties().forEach(prop => {
                    const propName = prop.getName() || '';
                    prop.getDecorators().filter(dec => {
                        const propDecName = dec.getName() || '';
                        if (propDecName === 'Input') {
                            component.inputs.push({
                                name: propName,
                                initialized: prop.getStructure()
                                    .initializer !== undefined
                            });
                        }
                        if (propDecName === 'Output') {
                            component.outputs.push({
                                name: propName,
                                initialized: prop.getStructure()
                                    .initializer !== undefined
                            });
                        }
                    });
                }));
                components.push(component);
            }
        });
    });
    return components;
}
exports.getAngularComponents = getAngularComponents;
